@@ -27,7 +27,7 @@ Lingua::EN::AddressParse - manipulate geographical addresses
     print $my_address{suburb};                  # ST JOHNS WOOD
 
     %my_formatted_address = $address->case_components;
-    print $my_formatted_address{street};        # Old South Head
+    print $my_formatted_address{street_name};        # Old South Head
     print $my_formatted_address{street_type};   # St
 
     %address_properties = $address->properties;
@@ -71,31 +71,29 @@ lists of postal addresses.
 
 =head1 DEFINITIONS
 
-The following terms are used by AddressParse to define
-the components that can make up an address.
+The following terms are used by AddressParse to define the components that
+can make up an address.
 
     Pre cursor : C/O MR A Smith...
-
     Sub property identifier : Level 1A Unit 2, Apartment B, Lot 12, Suite # 12 ...
     Property Identifier : 12/66A, 24-34, 2A, 23B/12C, 12/42-44
-
     Property name   : "Old Regret"
     Post Box        : GP0 Box K123, LPO 2345, RMS 23 ...
     Road Box        : RMB 24A, RMS 234 ...
+    Street Direction: North, SE, Sth. etc
     Street name     : O'Hare, New South Head, The Causeway
     Street type     : Road, Rd., St, Lane, Highway, Crescent, Circuit ...
     Suburb          : Dee Why, St. John's Wood ...
-    Sub country     : NSW, New South Wales, ACT, NY, AZ ...
-    Post (zip) code : 2062, 34532, SG12A 9ET
+    Sub country     : NSW, New South Wales, ACT, NY, New Jersey AZ ...
+    Post (zip) code : 2062, 34532-1234, SG12A 9ET
     Country         : Australia, UK, US or Canada
 
-Refer to the component grammar defined in the Lingua::EN::AddressParse::Grammar
-module for a complete list of combinations.
 
 
-The following address formats are currently supported. A ? means the component is optional:
+The main  address formats  currently supported are as follows. (a ? means the component is optional):
 
     'suburban' : sub_property_identifier(?) property_identifier(?) street street_type suburb subcountry post_code(?)country(?)
+
     OR for the USA
     'suburban' : property_identifier(?) street street_type sub_property_identifier(?) suburb subcountry post_code(?) country(?)
 
@@ -106,7 +104,8 @@ The following address formats are currently supported. A ? means the component i
 
 All formats may contain a precursor
 
-See Lingua::EN::AddressParse::Grammar for a complete list
+Refer to the component grammar defined in the Lingua::EN::AddressParse::Grammar
+module for a complete list of combinations.
 
 
 =head1 METHODS
@@ -140,7 +139,7 @@ optional argument to the C<new> method.
 The country argument must be specified. It determines the possible list of
 valid sub countries (states, counties etc, defined in the Locale::SubCountry
 module) and post code formats. Either the full name or abbreviation may be
-specified. The currently suppoted country names and codes are:
+specified. The currently supported country names and codes are:
 
     AU or Australia
     CA or Canada
@@ -160,11 +159,12 @@ When this option is set to a positive value, the input string is
 'cleaned' to try and normalise bad patterns. The type of cleaning
 includes
 
-    remove non alphanumeric characters.
+    remove non alphanumeric characters
+    remove full stops
     remove redundant white space
     add missing space separators
-    expand abbreviations to more common form
-    remove bracketed annnotations
+    expand abbreviations to more common forms
+    remove bracketed annotations
     fix badly formed sub property identifiers
 
 =item abbreviate_subcountry (optional)
@@ -178,7 +178,13 @@ already abbreviated then it's value is not altered.
 When this option is set to a positive value, only the abbreviated form
 of sub country is allowed, such as "NSW" and not "New South Wales". This
 will make parsing quicker and ensure that addresses comply with postal
-standards that normally permit ony abbrviated sub countries .
+standards that normally permit only abbreviated sub countries.
+
+It also avoids matching a sub_country name too early, as in the case of
+    'Port Washington New Jersey'
+Normally, 'Washington would be consumed as the sub country, but by first converting
+the address to 'Port Washington NJ' we avoid this problem
+
 
 =item force_post_code (optional)
 
@@ -209,19 +215,24 @@ This method is a prerequisite for all the following methods.
 =head2 components
 
     %address = $address->components;
-    $surburb = $address{suburb};
+    $suburb = $address{suburb};
 
-The C<components> method returns all the address componets in a hash. The
+The C<components> method returns all the address components in a hash. The
 following keys are used for each component:
 
+
+    pre_cursor - such as 'C/O Mr A Smith'
+    po_box_type - such as 'Private Boxes'
     post_box
     road_box
     sub_property_identifier
     property_identifier
     property_name
-    street
+    street_direction_prefix (such as East, NW, North etc)
+    base_street_name (the name with direction removed, such as "Main" in "East Main St")
+    street_name (the full street name such as "East Main")
     street_type
-    street_direction (US only)
+    street_direction_suffix (US only, abbreviated only such as N, SE etc)
     suburb
     subcountry
     post_code
@@ -230,7 +241,6 @@ following keys are used for each component:
 If a component has no matching data for a given address, it's values will be
 set to the empty string.
 
-
 =head2 case_components
 
    %my_address = $address->case_components;
@@ -249,6 +259,7 @@ The following components are not converted to title case:
     subcountry
     post_code
     country
+    street_direction_suffix
 
 =head2 case_all
 
@@ -258,7 +269,7 @@ The C<case_all> method does the same thing as the C<case_components> method exce
 the entire address is returned as a title cased text string.
 
 If the force_case option was set in the C<new> method above, address case the
-entire input string, including any unmatched sections after a recognzable address
+entire input string, including any unmatched sections after a recognisable address
 that failed parsing. This option is useful when you know you have invalid data,
 but you still want to title case what you have.
 
@@ -268,8 +279,16 @@ The C<properties> method return several properties of the address as a hash.
 The  following keys are used for each property -
 
     type - either suburban ,rural,post_box,road_box,unknown
+    non_matching - any trailing string not part the address
 
-    non_matching (any non matching portion of the input string)
+
+Additional properties can be accessed with the following
+
+  $address->{original_input}
+  $address->{input_string} - string after  auto_clean option has been applied
+  $address->{country_code} - abbreviated Country address format (as defined in the C<new> method)
+  $address->{error} - error flag, 0 = good, 1 = error
+  $address->{error_desc} - text to  to describe the type of parsing error
 
 
 =head2 report
@@ -281,7 +300,7 @@ Create a formatted text report
     the country type
     the address type
     any non matching part of input string
-    if any parsing errors occured
+    if any parsing errors occurred
     error description
 
     the name and value of each defined component
@@ -308,7 +327,10 @@ description.
 The huge number of character combinations that can form a valid address makes
 it is impossible to correctly identify them all.
 
-Valid addresses must contain: property address, suburb and subcountry (state) in that order.
+Valid addresses must contain:
+
+    property address, suburb,  subcountry (aka state) in that order.
+
 This format is widely accepted in Australia and the US.
 
 UK addresses will often include suburb, town, city and county, formats that
@@ -361,7 +383,7 @@ AddressParse was written by Kim Ryan <kimryan at cpan d o t org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014 Kim Ryan. All rights reserved.
+Copyright (c) 2015 Kim Ryan. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
@@ -380,7 +402,7 @@ use Lingua::EN::AddressParse::Grammar;
 use Lingua::EN::NameParse;
 use Parse::RecDescent;
 
-our $VERSION = '1.21';
+our $VERSION = '1.22';
 
 #------------------------------------------------------------------------------
 # Create a new instance of an address parsing object. This step is time
@@ -411,7 +433,7 @@ sub new
         $address->{$curr_key} = $args{$curr_key};
     }
 
-    # create the grammar tree (this is country dependant)
+    # create the grammar tree (this is country dependent)
     my $grammar = Lingua::EN::AddressParse::Grammar::_create($address);
 
     $address->{parse} = Parse::RecDescent->new($grammar);
@@ -427,7 +449,9 @@ sub parse
     # Save original data so we can check effect of auto cleaning
     $address->{original_input} = $input_string;
 
-    $address->{input_string} = $input_string;
+    # Convert to all upper case. This will allow for faster regexp matching in
+    # the grammar tree
+    $address->{input_string} = uc($input_string);
 
     chomp($address->{input_string});
 
@@ -448,9 +472,9 @@ sub parse
     }
 
     # We need to add a trailing space to the input string. This is because the grammar
-    # tree expects a terminator (the space) Ffor every production, optionally followed
+    # tree expects a terminator (the space) fro every production, optionally followed
     # by other productions or any final non matching text.
-    # This space will be removed by tye _assemble function
+    # This space will be removed in the _assemble function
     $address->{input_string} .= ' ';
 
     $address = _assemble($address,$pre_cursor,$po_box_type);
@@ -460,6 +484,7 @@ sub parse
     return($address,$address->{error});
 }
 #------------------------------------------------------------------------------
+# Return all address components as a hash
 sub components
 {
     my $address = shift;
@@ -478,12 +503,12 @@ sub case_components
     foreach my $curr_key ( keys %orig_components )
     {
         my $cased_value;
+        my $curr_value = $orig_components{$curr_key};
 
-        if ( $curr_key =~ /^(street|street_type|suburb|property_name|sub_property_identifier|pre_cursor|po_box_type)$/ )
+        if ( $curr_key =~ /^(base_street_name|street_name|street_type|suburb|property_name|sub_property_identifier|pre_cursor|po_box_type)$/ )
         {
-            # Surnames can be used for street's or suburbs so this method
-            # will give correct capitalisation for most cases
-            if ( $curr_key eq 'street' and $orig_components{$curr_key} =~ /^US Highway (.*)/i )
+
+            if ( $curr_key eq 'street_name' and$curr_value =~ /^US HIGHWAY (.*)/ )
             {
                 $cased_value = "US Highway $1";
             }
@@ -491,13 +516,13 @@ sub case_components
             {
                 # UNIT, APT ... 12D etc
 
-                my @words = split(/ /,$orig_components{$curr_key});
+                my @words = split(/ /,$curr_value);
                 my @cased_words;
                 my $cased_string;
                 foreach my $word (@words)
                 {
                     my $cased_word;
-                    if ( $word =~ /^\d{1,2}(st|nd|rd|th)$/i)
+                    if ( $word =~ /^\d{1,3}(ST|ND|RD|TH)$/)
                     {
                         # ordinal component, as in  3rd Floor
                         $cased_word = lc($word);
@@ -518,14 +543,35 @@ sub case_components
             }
             else
             {
-                $cased_value = Lingua::EN::NameParse::case_surname($orig_components{$curr_key});
+                if ($curr_value)
+                {
+                    # Surnames can be used for street's or suburbs so this method
+                    # will give correct capitalisation for most cases
+                    $cased_value = Lingua::EN::NameParse::case_surname($curr_value);
+                }
+                else
+                {
+                    $cased_value = '';
+                }
             }
-
         }
-        # retain street_direction,sub country and countries capitalisation, usually uppercase
+       # retain street_direction,sub country and countries capitalisation, usually uppercase
+        elsif ($curr_key =~ /street_direction/)
+        {
+            if (length($curr_value) == 1 or length($curr_value) == 2)
+            {
+                # N, SE etc is capitalised
+                $cased_value =$curr_value;
+            }
+            else
+            {
+                $cased_value = Lingua::EN::NameParse::case_surname($curr_value);
+            }
+        }
+        # retain sub country and countries capitalisation, as usually uppercase
         else
         {
-            $cased_value = uc($orig_components{$curr_key});
+            $cased_value = uc($curr_value);
         }
         $cased_components{$curr_key} = $cased_value;
     }
@@ -555,7 +601,7 @@ sub case_all
         );
         if ( $address->{country} eq 'US' )
         {
-           $component_order{'suburban'} = [ qw/pre_cursor property_identifier street street_type street_direction sub_property_identifier suburb subcountry post_code country/];
+           $component_order{'suburban'} = [ qw/pre_cursor property_identifier street street_type street_direction_suffix sub_property_identifier suburb subcountry post_code country/];
         }
         else
         {
@@ -616,6 +662,7 @@ sub report
         _fmt_report_line(\$report,"Address type",$props{type});
     }
 
+
     _fmt_report_line(\$report,"Non matching part",$props{non_matching});
     _fmt_report_line(\$report,"Error",$address->{error});
     _fmt_report_line(\$report,"Error descriptions",$address->{error_desc});
@@ -635,22 +682,12 @@ sub report
     return($report);
 }
 
-#-------------------------------------------------------------------------------
-
-sub _fmt_report_line
-{
-    my ($report_ref,$label,$value) = @_;
-    $$report_ref .= sprintf("%-23.23s '%s'\n",$label,$value);
-}
 
 #------------------------------------------------------------------------------
 
 # PRIVATE METHODS
 
 #------------------------------------------------------------------------------
-# Parse the address
-# Assemble all the components of the address into an object
-# Remove trailing white space
 
 sub _assemble
 {
@@ -718,23 +755,39 @@ sub _assemble
         $address->{components}{property_identifier} = _trim_trailing_space($parsed_address->{property_identifier});
     }
 
-    $address->{components}{street} = '';
-    if ( $parsed_address->{street} )
+    $address->{components}{base_street_name} = '';
+    $address->{components}{street_direction_prefix} = '';
+    my ($street_direction,$base_street_name) = _get_street_direction($parsed_address->{street_name});
+    if ($street_direction )
     {
-        $address->{components}{street} = _trim_trailing_space($parsed_address->{street});
+        $address->{components}{street_direction_prefix} =  $street_direction;
+        $address->{components}{base_street_name} = $base_street_name;
     }
 
+    $address->{components}{street_name} = '';
     $address->{components}{street_type} = '';
-    if ( $parsed_address->{street_type} )
+    if ( $parsed_address->{street_name} )
     {
-        $address->{components}{street_type} =  _trim_trailing_space($parsed_address->{street_type});
+        # Streets such as 'The Corso' will parse as street_name = 'The' and street_type = 'Corso', so seperate out
+        if ( $parsed_address->{street_name} eq 'THE ' )
+        {
+             $address->{components}{street_name} = 'THE ' .  _trim_trailing_space($parsed_address->{street_type});
+        }
+        else
+        {
+            $address->{components}{street_name} = _trim_trailing_space($parsed_address->{street_name});
+            $address->{components}{street_type} =  _trim_trailing_space($parsed_address->{street_type});
+        }
     }
-    $address->{components}{street_direction} = '';
-    if ( $parsed_address->{street_direction} )
+
+
+    $address->{components}{street_direction_suffix} = '';
+    if ( $parsed_address->{street_direction_suffix} )
     {
-        $address->{components}{street_direction} =  _trim_trailing_space($parsed_address->{street_direction});
+        $address->{components}{street_direction_suffix} =  _trim_trailing_space($parsed_address->{street_direction_suffix});
     }
 
+
     $address->{components}{suburb} = '';
     if ( $parsed_address->{suburb} )
     {
@@ -791,6 +844,35 @@ sub _assemble
 
     return($address);
 }
+
+#-------------------------------------------------------------------------------
+#
+sub _get_street_direction
+{
+    my ($street_name) = @_;
+
+    my $street_direction;
+    my $base_street_name;
+
+    unless ($street_name)
+    {
+        return;
+    }
+
+    my @words = split(/\s/,$street_name);
+    if (@words > 1)
+    {
+        if  ( $words[0] =~ /^(N|NE|NW|E|S|SE|SW|W|NORTH|EAST|SOUTH|WEST|NTH|STH)$/ )
+        {
+            $street_direction = $1;
+            shift(@words);
+            $base_street_name = join(' ',@words);
+        }
+    }
+    return($street_direction,$base_street_name);
+
+}
+
 #------------------------------------------------------------------------------
 # Check for several different types of syntax errors
 
@@ -821,7 +903,7 @@ sub _validate
             }
 
             # illegal characters found, note a '#' can appear as an abbreviation for number in USA addresses
-            if ( $address->{input_string} =~ /[^"A-Za-z0-9'\-\.,&#\/ ]/ )
+            if ( $address->{input_string} =~ /[^"A-Z0-9'\-\.,&#\/ ]/ )
             {
                 # Note, if auto_clean is on, illegal characters will have been removed
                 # for second parsing and no error flag or message reported
@@ -830,11 +912,11 @@ sub _validate
             }
             if ( $address->{properties}{type} eq 'suburban' )
             {
-                my $street = $address->{components}{street};
+                my $street = $address->{components}{street_name};
                 if ($street !~ /\d/ and length($street) > 1 )
                 {
-                    # Not and ordinal or single letter street type
-                    if ( $address->{components}{street} !~ /[AEIOUY]/i )
+                    # Not an ordinal or single letter street type
+                    if ( $address->{components}{street_name} !~ /[AEIOUY]/ )
                     {
                         # street name must have a vowel sound,
                         $address->{error} = 1;
@@ -843,7 +925,7 @@ sub _validate
                 }
             }
 
-            if ( $address->{components}{suburb} !~ /[AEIOUY]/i )
+            if ( $address->{components}{suburb} !~ /[AEIOUY]/ )
             {
                 $address->{error} = 1;
                 $address->{error_desc} .= ':no vowel sound in suburb';
@@ -869,7 +951,7 @@ sub _clean
     # hash (#) may denote number for USA address
     # quotes can occur as property name delimiters
 
-    $input =~ s|[^A-Za-z0-9&#/.'" -]||go;
+    $input =~ s|[^A-Za-z0-9&#/'" -]||go;
 
     # remove repeating, leading and trailing spaces
     $input =~ s|  +| |go ;
@@ -878,22 +960,18 @@ sub _clean
 
     # Expand abbreviations that are too short
 
-     # street nouns
+    # street nouns
     $input =~ s| CSEWY | CAUSEWAY |;
-    $input =~ s| Csewy | Causeway |;
 
     # street types
 
 
     $input =~ s|^FCTR?Y |FACTORY |;
-    $input =~ s|^Fctr?y |Factory |;
     $input =~ s|^FACT?R?Y? |FACTORY |;
-    $input =~ s|^Fact?r?y? |Factory |;
 
     $input =~ s|LVL |LEVEL |; # sub property identifiers
-    $input =~ s|Lvl |Level |;
-    $input =~ s|^UN? |Unit |;
-    $input =~ s|^U(\d+)|Unit $1|;
+    $input =~ s|^UN? |UNIT |;
+    $input =~ s|^U(\d+)|UNIT $1|;
 
 
     # Fix badly formed number dividers such as home unit format of 14/ 12 becomes 14/12, 2- 7A becomes 2-7A
@@ -904,7 +982,7 @@ sub _clean
 
     # Remove redundant spaces in property identifiers,  21 B Smith St becomes 21B Smith St
 
-    if ( $input !~ /^\d+ [A-Z] S[T|t]/ )
+    if ( $input !~ /^\d+ [A-Z] (ST|AVE)/ )
     {
         # Don't remove space before single letter streets such as 21 B Street
         if ( $address->{country_code} eq 'US' )
@@ -932,19 +1010,20 @@ sub _clean
         $input =~ s| # (\d+) (\w) | # $1$2 |;
 
         # remove redundnant '#'
-         $input =~ s| APT #| Apt |i;
+         $input =~ s| APT #| APT |;
     }
     else
     {
         # Add a space to separate sub property type from number, UNIT2 becomes UNIT 2
-        $input =~ s/^(Unit|Lot|Apt|Shop)(\d)/$1 $2/i;
+        $input =~ s/^(UNIT|LOT|APT|SHOP)(\d)/$1 $2/;
     }
 
     # Remove redundant slash or dash
     # Unit 1B/22, becomes Unit 1B 22, Flat 2-12 becomes Flat 2 12
-    $input =~ s/^([A-Za-z]{2,}) (\d+[A-Z]?)[\/-]/$1 $2 /i;
+    # TO DO, add |# at start
+    $input =~ s/^([A-Z]{2,}) (\d+[A-Z]?)[\/-]/$1 $2 /;
     # Unit J1/ 39 becomes  Unit J1 39
-    $input =~ s/^([A-Za-z]{2,}) ([A-Z]\d{0,3})[\/-]/$1 $2 /i;
+    $input =~ s/^([A-Z]{2,}) ([A-Z]\d{0,3})[\/-]/$1 $2 /;
 
 
     # remove dash that is not from a sequence, such as D-5 or 22-A
@@ -963,7 +1042,7 @@ sub _extract_precursor
     my ($input) = @_;
     my ($pre_cursor,$address_start,$address_end);
 
-    if ($input =~ m{^(C/O.*?|Attn.*?) (\d+|PO BOX)( .*)}i)
+    if ($input =~ m{^(C/O.*?|ATTN.*?) (\d+|PO BOX)( .*)})
     {
         $pre_cursor = $1;
         $address_start = $2;
@@ -985,7 +1064,7 @@ sub _extract_po_box_type
     my ($input) = @_;
     my ($po_box_type,$address_start,$address_end);
 
-    if ($input =~ /^(.*? )(Private Boxes)( .*)$/i )
+    if ($input =~ /^(.*? )(PRIVATE BOXES)( .*)$/ )
     {
         $address_start = $1;
         $po_box_type = $2;
@@ -1006,7 +1085,11 @@ sub _trim_trailing_space
     $string =~ s/ $//;
     return($string);
 }
-
-
 #------------------------------------------------------------------------------
+
+sub _fmt_report_line
+{
+    my ($report_ref,$label,$value) = @_;
+    $$report_ref .= sprintf("%-23.23s '%s'\n",$label,$value);
+}
 return(1);
