@@ -18,11 +18,11 @@ for more details.
 
 =head1 AUTHOR
 
-AddressGrammar was written by Kim Ryan, kimryan at cpan d-o-t or g
+Lingua::EN::AddressParse::Grammar was written by Kim Ryan, kimryan at cpan d-o-t or g
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014 Kim Ryan. All rights reserved.
+Copyright (c) 2015 Kim Ryan. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
@@ -35,7 +35,7 @@ use strict;
 use warnings;
 use Locale::SubCountry;
 
-our $VERSION = '1.21';
+our $VERSION = '1.22';
 
 #-------------------------------------------------------------------------------
 # Rules that define valid orderings of an addresses components
@@ -47,8 +47,8 @@ my $non_usa_suburban_address_rules =
 q{
     full_address :
 
-    # Note: both sub property and property indetifiers should be optional. This
-    # will allow for cases such as 'Lot 123 Xyz Street' where Lot is in effect the house number, even though 'Lot' is grouped as a sub_property lablel
+    # Note: both sub property and property identifiers should be optional. This
+    # will allow for cases such as 'Lot 123 Xyz Street' where Lot is in effect the house number, even though 'Lot' is grouped as a sub_property label
     # Also, cases such as 'SHOP 12A, CHAPEL RD STH' have no street number so are incomplete, but still may need to be parsed
 
     sub_property_identifier(?) property_identifier(?) street_untyped suburb subcountry post_code(?) country(?) non_matching(?)
@@ -62,7 +62,8 @@ q{
             # an anonymous hash reference
             sub_property_identifier => $item[1][0],
             property_identifier     => $item[2][0],
-            street                  => $item[3],
+            street_name             => $item[3],
+            street_type             => '',
             suburb                  => $item[4],
             subcountry              => $item[5],
             post_code               => $item[6][0],
@@ -79,7 +80,7 @@ q{
         {
             sub_property_identifier => $item[1][0],
             property_identifier     => $item[2][0],
-            street                  => $item[3],
+            street_name             => $item[3],
             street_type             => $item[4],
             suburb                  => $item[5],
             subcountry              => $item[6],
@@ -107,7 +108,7 @@ q{
         $return =
         {
             property_identifier     => $item[1][0],
-            street                  => $item[2],
+            street_name             => $item[2],
             street_type             => '',
             sub_property_identifier => $item[3][0],
             suburb                  => $item[4],
@@ -120,14 +121,14 @@ q{
     }
     |
 
-    property_identifier(?) street street_type street_direction(?) sub_property_identifier(?) suburb subcountry post_code(?) country(?) non_matching(?)
+    property_identifier(?) street street_type abbrev_direction(?) sub_property_identifier(?) suburb subcountry post_code(?) country(?) non_matching(?)
     {
         $return =
         {
             property_identifier     => $item[1][0],
-            street                  => $item[2],
+            street_name             => $item[2],
             street_type             => $item[3],
-            street_direction        => $item[4][0],
+            street_direction_suffix => $item[4][0],
             sub_property_identifier => $item[5][0],
             suburb                  => $item[6],
             subcountry              => $item[7],
@@ -150,7 +151,7 @@ q{
         {
            property_name       => $item[1],
            property_identifier => $item[2],
-           street              => $item[3],
+           street_name         => $item[3],
            street_type         => $item[4],
            suburb              => $item[5],
            subcountry          => $item[6],
@@ -166,7 +167,7 @@ q{
         $return =
         {
            property_name       => $item[1],
-           street              => $item[2],
+           street_name         => $item[2],
            street_type         => $item[3],
            suburb              => $item[4],
            subcountry          => $item[5],
@@ -220,7 +221,7 @@ q{
         $return =
         {
            road_box      => $item[1],
-           street        => $item[2],
+           street_name   => $item[2],
            street_type   => $item[3],
            suburb        => $item[4],
            subcountry    => $item[5],
@@ -271,11 +272,11 @@ q{
     # Level 4 Tower A,
     # Unit 3A Block C
     sub_property_identifier :
-        /Level (\d+|[GM]) (Building|Tower) \d+[A-Z]? /i |
-        /Level (\d+|[GM]) (Building|Tower) [A-Z] /i |
-        /Unit \d+[A-Z]? (Block|Bldg|Building|Tower) ([A-Z]|\d+) /i |
-        /\d{1,2}(st|nd|rd|th) (Floor|Flr|Fl) /i |
-        /(Floor|Flr|Fl) \d{1,2} /i
+        /LEVEL (\d+|[GM]) (BUILDING|TOWER) \d+[A-Z]? / |
+        /LEVEL (\d+|[GM]) (BUILDING|TOWER) [A-Z] / |
+        /UNIT \d+[A-Z]? (BLOCK|BLDG|BUILDING|TOWER) ([A-Z]|\d+) / |
+        /\d{1,2}(ST|ND|RD|TH) (FLOOR|FLR|FL) / |
+        /(FLOOR|FLR|FL) \d{1,2} /
 
         |
 
@@ -287,44 +288,49 @@ q{
     # Unit 34, BLDG 12, SUITE A,
 
     sub_property_name:
-        /Apartment /i  | /Apt /i   |
-        /Bay /i        |
-        /Building /i   | /Bldg /i  |
-        /Department /i |
-        /Factory /i    |
-        /Flat /i       |
-        /Front /i      |
-        /Frnt /i       |
-        /Gate /i       |
-        /Key /i        |
-        /Hangar /i     | /Hngr /i  |
-        /Key /i        |
-        /Lobby /i      |
-        /Lbby /i       |
-        /Lot /         |
-        /Office /i     |
-        /Ofc /i        |
-        /Level /i      |
-        /Lot /i        |
-        /No\.? /i     |
-        /Penthouse /i  |
-        /Ph /i         |
-        /Pier /i       |
-        /Rear (Of )?/i |
-        /Room /i       |
-        /Rm /i         |
-        /Shop /i       |
-        /Shed /i       |
-        /Suite /i      | /Ste /i |
-        /Trailer /i    |
-        /Trlr /i       |
-        /Unit /i       |
-        /Villa /i      |
-        /# /               # Note '#' is a common abbreviation for number in USA
+        /(
+        APARTMENT   | APT   |
+        BAY         |
+        BUILDING    | BLDG  |
+        DEPARTMENT  |
+        FACTORY     |
+        FLAT        |
+        FRONT       |
+        FRNT        |
+        GATE        |
+        KEY         |
+        HANGAR      | HNGR  |
+        KEY         |
+        LOBBY       |
+        LBBY        |
+        LOT         |
+        OFFICE      |
+        OFC         |
+        LEVEL       |
+        LOT         |
+        NO          |
+        PENTHOUSE   |
+        PH          |
+        PIER        |
+        REAR (OF )? |
+        ROOM        |
+        RM          |
+        SHOP        |
+        SHED        |
+        SUITE       | STE |
+        TRAILER     |
+        TRLR        |
+        UNIT        |
+        VILLA       |
+        \#                # Note '#' is a common abbreviation for number in USA
+        )\ /x
 
     sub_property_number :
-        /\d{1,6}[A-Z]? /i | # such as 23B
-        /[A-Z]\d{0,3} /     # such as # D512
+        /\d{1,6}[A-Z]{0,2} / | # such as 23B, 6AW
+        /\d{1,2}[A-Z]\d /    | # such as 4A5
+        /[A-Z]{1,2}\d{0,4} / | # such as # D512 # changed for Erik
+        /\d{1,3}-\d{1,3} /     # such as 114-234. 12-33 can be building 1 floor 14  unit 234
+
 
 };
 
@@ -347,8 +353,8 @@ q{
 my $property_name =
 q{
     # Property or station names like "Old Regret" or 'Never Fail'
-    property_name : /\"[A-Z'-]{2,}( [A-Z'-]{2,})?\" /i |
-                    /\'[A-Z-]{2,}( [A-Z-]{2,})?\' /i
+    property_name : /\"[A-Z'-]{2,}( [A-Z'-]{2,})?\" / |
+                    /\'[A-Z-]{2,}( [A-Z-]{2,})?\' /
 };
 #------------------------------------------------------------------------------
 
@@ -360,15 +366,16 @@ q{
         $return = "$item[1]$item[2]"
     }
 
+    # NOTE: extended regexps not useful here, too many spaces to delimit
     post_box_type :
-        /G\.?P\.?O\.? Box /i |
-        /L\.?P\.?O\.? Box /i |
-        /P ?O Box /i         |
-        /P\.?O\.? Box /i     |
-        /Locked Bag /i       |
-        /Private Bag /i
+        /GPO BOX /  |
+        /LPO BOX /  |
+        /P ?O BOX / |
+        /PO BOX /   |
+        /LOCKED BAG /   |
+        /PRIVATE BAG /
 
-    post_box_number : /[A-Z]?\d{1,6}[A-Z]? /i
+    post_box_number : /[A-Z]?\d{1,6}[A-Z]? /
 };
 #------------------------------------------------------------------------------
 
@@ -381,14 +388,16 @@ q{
     }
 
     road_box_type :
-        /CMB / | # Community Mail Bag
-        /CMA / | # Community Mail Agent
-        /CPA / | # Community Postal Agent
-        /RMS / | # Roadside Mail Service
-        /RMB / | # Roadside Mail Box
-        /RSD /   # Roadside Side Delivery
+        /(
+        CMB | # Community Mail Bag
+        CMA | # Community Mail Agent
+        CPA | # Community Postal Agent
+        RMS | # Roadside Mail Service
+        RMB | # Roadside Mail Box
+        RSD   # Roadside Side Delivery
+        )\ /x  # note space separator needed at end of token
 
-    road_box_number : /[A-Z]?\d{1,5}[A-Z]? /i
+    road_box_number : /[A-Z]?\d{1,5}[A-Z]? /
 
 };
 
@@ -402,93 +411,27 @@ q{
     street_untyped :
 
         major_road |
-        street_noun |
         avenue_letter |
-        street_name_single_word
-
+        street_name_single_word |
+        /RUE (DE |DES )?/ any_word
+        {
+            $return = "$item[1]$item[2]"
+        }
+        |
+        /AVENUE OF THE AMERICAS /
 
     major_road :
-        /(State |US )?(Highway|Hwy) \d{1,3} ([N|E|S|W] )?/i |
-        /(State|County) Road \d{1,3} ([N|E|S|W] )?/i |
-        /(US |State |County )?Route \d{1,3} ([N|E|S|W] )?/i |
-        /Alt \d{1,3} ([N|E|S|W] )?/i  # Alternate road
-
-   # allow for case where street type IS the street name, as in The PARADE
-
-    street_noun:
-        /The /i nouns { $return = "$item[1]$item[2]" }
-
-        nouns:
-        /Arcade /i       |
-        /Avenue /i       |
-        /Battlement /i   |
-        /Boulevarde? /i  |
-        /Broadwater /i   |
-        /Broadway /i     |
-        /Cascades /i     |
-        /Carriageway /i  |
-        /Causeway /i     |
-        /Centre /i       |
-        /Chase /i        |
-        /Circle /i       |
-        /Circuit /i      |
-        /Close /i        |
-        /Concord /i      |
-        /Corniche /i     |
-        /Coronado /i     |
-        /Corso /i        |
-        /Crest /i        |
-        /Crossover /i    |
-        /Crescent /i     |
-        /Dell /i         |
-        /Deviation /i    |
-        /Driftway /i     |
-        /Entrance /i     |
-        /Esplanade /i    |
-        /Fairway /i      |
-        /Glade /i        |
-        /Glen /i         |
-        /Grange /i       |
-        /Greenway /i     |
-        /Grove /i        |
-        /Haven /i        |
-        /Kingsway /i     |
-        /Knoll /i        |
-        /Lanterns /i     |
-        /Mall /i         |
-        /Mainbrace /i    |
-        /Mainsail /i     |
-        /Overflow /i     |
-        /Oval /i         |
-        /Parade /i       |
-        /Portico /i      |
-        /Parkway /i      |
-        /Peninsula /i    |
-        /Plaza /i        |
-        /Promenade /i    |
-        /Rampart /i      |
-        /Retreat /i      |
-        /Ridge /i        |
-        /Row /i          |
-        /Serpentine /i   |
-        /Square /i       |
-        /Strand /i       |
-        /Terrace /i      |
-        /Walk /i         |
-
-        /Five Ways /i    |
-        /Six Ways /i     |
-        /Seven Ways /i   |
-        /Eight Ways /i   |
-        /Nine Ways /i
+        /(COUNTY |STATE |US )?(ALT|HIGHWAY|HWY|ROAD|RD|ROUTE) \d{1,3} ([N|E|S|W|NORTH|EAST|SOUTH|WEST] )?/
 
     avenue_letter :
-        /Avenue [A-Z]/i
+        /AVENUE [A-Z] /
 
     street_name_single_word:
-        /([N|E|S|W] )?Broadway /i |
-        /Kingsway /i |
-        /Queensway /i
+        /([N|E|S|W] )?BROADWAY|BOARDWALK|BOULEVARD|BOWERY|ESPLANADE|KINGSWAY|QUEENSWAY|GREENWAY|PARKWAY|RIVERBANK /
+        ...!street_type
+        {
+            $return = $item[1]
+        }
 
     #----------------------------------------------------------------------------
 
@@ -511,54 +454,89 @@ q{
         }
     }
 
+
     prefix :
-
-        /New /i       |
-        /Old /i       |
-        /Mt\.? /i     |
-        /Mount /i     |
-        /Dame /i      |
-        /Sir /i       |
-
-        /North /i     |
-        /Nth\.? /i    |
-        /N\.? /       |
-        /NE\.? /      |
-        /NW\.?/       |
-        /East /i      |
-        /E\.? /       |
-        /South /i     |
-        /Sth\.? /i    |
-        /S\.? /       |
-        /SE\.? /      |
-        /SW\.? /      |
-        /West /i      |
-        /W\.? /       |
-
-        /Upper /i     |
-        /U\.? /i      |
-        /Lower /i     |
-        /L\.? /i
-
+        direction |
+        /(
+        NEW|
+        OLD|
+        MT|
+        MOUNT|
+        DAME|
+        SIR|
+        U|UPPER|
+        L|LOWER|
+        LA
+        )\ /x
 
     street_name :
 
-        /(N |E |S |W |Dr )?(Martin Luther|ML) King ([JS]r )?/i
-        {
-            $return = "$item[1]"
-        }
+        /(N |E |S |W |DR )?(MARTIN LUTHER|MARTIN L|ML) KING ([JS]R )?/
+        |
+        street_name_ordinal
         |
 
+        # WORD STREET_TYPE STREET_TYPE
         # Queen's Park Road, Grand Ridge Rd, Terrace Park Drive, Lane Cove Road etc
-        any_word /(Burn|Cay|Cove|Garden|Glen|Grove|Island|Isle|Key|Loop|Pass|Park|Parkway|Ridge|View) /i ...street_type
+        any_word
+        /(
+        BEND|
+        BURN|
+        CAY|
+        CHASE|
+        CLUB|
+        CROSS|
+        COVE|
+        GARDEN|
+        GLEN|
+        GROVE|
+        HAVEN|
+        HILL|
+        HOLLOW|
+        ISLAND|
+        ISLE|
+        KEY|
+        LANDING|
+        LANE|
+        LOOP|
+        PASS|
+        PARK|
+        PATH|
+        PARKWAY|
+        PLACE|
+        POINT|
+        POINTE|
+        RUN|
+        RIDGE|
+        SQUARE|
+        VIEW|
+        VISTA
+        )\ /x
+        ...street_type
         {
             $return = "$item[1]$item[2]"
         }
-
         |
 
-        # Glen Alpine Way, La Boheme Ave, St. Kilda Rd, Grove Valley Ave, Green Bay Road
-        /(Crescent|Glen|Green|Grove|Key|La|Lt\.?|Park|St\.?) /i street_name_word ...street_type
+        # STREET_TYPE WORD STREET_TYPE
+        # Glen Alpine Way, La Boheme Ave,  Grove Valley Ave, Green Bay Road
+        /(
+        CLUB|
+        CRESCENT|
+        CROSS|
+        GLEN|
+        GREEN|
+        GROVE|
+        FAIRWAY|
+        HILL|
+        KEY|
+        LA|
+        LT|
+        PARK|
+        PLAZA|
+        ST
+        )\ /x
+        street_name_word ...street_type
         {
             $return = "$item[1]$item[2]"
         }
@@ -578,8 +556,6 @@ q{
         |
         street_name_words
         |
-        street_name_ordinal
-        |
         street_name_letter
 
 
@@ -605,98 +581,154 @@ q{
     # Street in Green Street
     # Even two letter streets such as 'By Street' are valid
 
-    street_name_word: ...!street_type /[A-Z'-]{2,}\s+/i
+    street_name_word: ...!street_type /[A-Z'-]{2,}\s+/
     {
         $return = $item[2]
     }
 
 
-    # eg 42nd Street
-    street_name_ordinal:
-        /\d{0,2}1st\s+/i    |
-        /\d{0,2}2nd\s+/i    |
-        /\d{0,2}3rd\s+/i    |
-        /\d{0,2}[4-9]th\s+/i |
-        /\d{0,2}0th\s+/i    |
-        /\d{0,1}11th\s+/i   |
-        /\d{0,1}12th\s+/i   |
-        /\d{0,1}13th\s+/i
+    # eg Bay 12th Ave, 42nd Street
+    street_name_ordinal :
+        any_word(?)
+        /(
+        \d{0,2}1ST    |
+        \d{0,2}2ND    |
+        \d{0,2}3RD    |
+        \d{0,2}[4-9]TH |
+        \d{0,2}0TH    |
+        \d{0,1}11TH   |
+        \d{0,1}12TH   |
+        \d{0,1}13TH
+        )\ /x
+    {
+
+        if ( $item[1][0] and $item[2] )
+        {
+           $return = "$item[1][0]$item[2]"
+        }
+        elsif ($item[2] )
+        {
+           $return = "$item[2]"
+        }
+    }
 
 
     street_name_letter:  /[A-Z]\s+/  # eg B (Street)
 
     street_type:
 
-         # Place most common types first to improve the speed of matching
-         /St\.? /i    | /Street /i       |
-         /Rd\.? /i    | /Road /i         |
-         /La\.? /i    | /Lane /i         | /Ln /i |
-         /Ave?\.? /i  | /Avenue /i       |
-         /Al\.? /i    | /Alley /i        |
-         /Arc\.? /i   | /Arcade /i       |
-         /Bayway /i   |
-         /Bvd\.? /i   | /Blvd?\.? /i     | /Boulevarde? /i  |
-         /Bnd\.? /i   | /Bend /i         |
-         /Bl\.? /i    | /Bowl /i         |
-         /Br\.? /i    | /Brae /i         |
-         /Brow /i      |
-         /Cay /i      |
-         /Cir\.? /i   | /Circle /i       | /Crcle /i   |
-         /Cct\.? /i   | /Crt\.? /i       | /Cir\.? /i  | /Circuit /i |
-         /Cl\.? /i    | /Close /i        |
-         /Ct\.? /i    | /Court /i        |
-         /Cove /i     |
-         /Cres\.? /i  | /Crs\.? /i       | /Cr\.? /i   | /Crescent /i |
-         /Crest /i    |
-         /Dr\.? /i    | /Drv\.? /i       | /Drive /i   |
-         /Esp\.? /i   | /Esplanade /i    |
-         /Exp\.? /i   | /Expressway /i   |
-         /Fw?y\.? /i  | /Freeway /i      |
-         /Gln\.? /i   | /Glen /i         |
-         /Green /i    | /Grn /i          |
-         /Gr\.? /i    | /Grove /i        |
-         /Hwa?y\.? /i | /Highway /i      |
-         /Island /i   | /Is /i           | /Isle? /i     |
-         /Key /i      |
-         /Loop /i     |
-         /Mall /i     |
-         /Mews /i     |
-         /Pass /i     |
-         /Pasage /i   | /Psge /i         | /Psg /i     |
-         /Path /i     |
-         /Pde\.? /i   | /Parade /i       |
-         /Pk\.? /i    | /Park /i         |
-         /Parkway /i  | /Pkwy\.? /i      |
-         /Pike /i     |
-         /Pl\.? /i    | /Place /i        |
-         /Plz\.? /i   | /Plaza /i        |
-         /Ramble /i   |
-         /Rdg\.? /i   | /Ridge /i        |
-         /Ride /i     | /Rde /i          |
-         /Rise /i     | /Rse /i          |
-         /Run /i      |
-         /Rdy\.? /i   | /Roadway /i      |
-         /Row /i      |
-         /Sq\.? /i    | /Square /i       |
-         /Tce\.? /i   | /Ter /i          | /Trce /i    | /Terrace /i |
-         /Trl /i      | /Trail /i        |
-         /Tpke\.? /i  | /Turnpike /i     |
-         /Turn /i     |
-         /Throughway /i  |
-         /Wl?k\.? /i  | /Walk /i         |
-         /Wy\.? /i    | /Way /i          | /Wynde /i
+        /(
+        # Place most frequent types first to speed up matching
+        ST     |STREET         |
+        RD     |ROAD           |
+        LA     |LN | LANE      |
+        AVE?   | AVENUE        |
 
+        ALY?   | ALLEY         |
+        ARC    | ARCADE        |
+        BATTLEMENT |
+        BROADWATER |
+        BAYWAY    |
+        BVD    | BLVD?      | BOULEVARDE?   |
+        BND    | BEND          |
+        BL     | BOWL          |
+        BR     | BRAE          |
+        BROW      |
+        CASCADES  |
+        CAY       |
+        CARRIAGEWAY   | # typically appears as 'The Carriageway', a street noun
+        CAUSEWAY      | # a street noun
+        CENTRE        |
+        CONCORD       | # a street noun
+        CONCOURSE     |
+        CORNICHE      | # a street noun
+        CORONADO      | # a street noun
+        CORSO         | # a street noun
+        CIR       | CIRCLE   | CRCLE    |
+        CCT       | CRT      | CIR   | CIRCUIT  |
+        CHASE|
+        CL        | CLOSE      |
+        CROSS     | CROSSOVER  | CROSSING |
+        CR?T      | COURT      |
+        CV        | COVE       |
+        CRES      | CRS        | CR    | CRESCENT  |
+        CREST     |
+        CROFT     |
+        DELL      |
+        DEVIATION |
+        DRIFTWAY  |
+        DR        | DRV        | DRIVE|
+        ENCLOSURE |
+        ENTRANCE  |
+        ESP       | ESPLANADE     |
+        EXP       | EXPW?Y        | EXPRESSWAY |
+        FAIRWAY   |
+        FENWAY    | # a street noun
+        FELLSWAY  | # a street noun
+        FW?Y      | FREEWAY|
+        GLADE     |
+        GRANGE    |
+        GREENWAY  | # a street noun
+        GLN       | GLEN|
+        GREENS?   | GRN|
+        GR        | GROVE|
+        HAVEN     |
+        HL        | HILL|
+        HWA?Y     | HIGHWAY|
+        HOLLOW    |
+        ISLE?     | IS         | ISLAND |
+        KEY       |
+        KINGSWAY  | # a street noun
+        KNOLL     |
+        LANTERNS  |
+        LANDING   |
+        LOOP      |
+        MAINBRACE | # a street noun
+        MAINSAIL  | # a street noun
+        MALL      | # a street noun
+        MEWS      |
+        OVERFLOW  |
+        OVAL      |
+        PASS      |
+        PASSAGE   | PSGE          | PSG|
+        PATH      |
+        PDE       | PARADE|
+        PK        | PARK|
+        PARKWAY   | PKWY|
+        PENINSULA |
+        PIERS     |
+        PIKE      |
+        PL        | PLACE|
+        PLZ       | PLAZA|
+        PORTICO   |
+        PROMENADE |
+        PT        | POINTE?|
+        RAMBLE    |
+        RAMPART   | # a street noun
+        RDG       | RIDGE         |
+        RETREAT   | # a street noun
+        RIDE      | RDE           |
+        RISE      | RSE           |
+        ROTUNDA   | # a street noun
+        RUN       |
+        RDY       | ROADWAY       |
+        ROW       |
+        SERPENTINE | # a street noun
+        STRAND    | # a street noun
+        SLIP      |
+        SQ        | SQUARE|
+        TCE       | TER           | TRCE     | TERRACE  |
+        TRL       | TRAIL|
+        TPKE      | TURNPIKE|
+        TURN|
+        THROUGHWAY|
+        VLG | VILLAGE |
+        WL?K   | WALK|
+        WY     | WAY| WYNDE |
+        WAYS  # such as in 'The Five Ways'
 
-    street_direction:
+        )\ /x  # note space separator needed at end of token
 
-        /N /  |
-        /NE / |
-        /NW / |
-        /E /  |
-        /S /  |
-        /SE / |
-        /SW / |
-        /W /
 
 };
 
@@ -710,8 +742,10 @@ q
     suburb_prefix :
 
         prefix   |
-        /Lake /i
+        /CAPE / |
+        /LAKE /
 
+   # suburb: prefix? any_word suburb_word(0..2)
 
     suburb:
 
@@ -729,7 +763,7 @@ q
         }
         |
 
-        # such as  Victoria Valley, Concord West
+        # such as  Washington Valley, Lane Cove West
         any_word suburb_word(0..2)
         {
             if ( $item[2][0] and $item[2][1]  )
@@ -747,7 +781,7 @@ q
         }
         |
         # such as Kippa-ring or Brighton-Le-Sands
-        /[A-Z]{2,}-[A-Z]{2,}(-[A-Z]{2,})?/i
+        /[A-Z]{2,}-[A-Z]{2,}(-[A-Z]{2,})?/
 
     suburb_word: ...!subcountry any_word
 };
@@ -758,10 +792,35 @@ q
 {
     # For use in first or second word of double or triple word street names or suburbs
     # such as  Moore Park West
-    any_word: /[A-Z'-]{2,}\s+/i
+    any_word: /[A-Z'-]{2,}\s+/
     {
         $return = $item[1]
     }
+
+    direction: full_direction | abbrev_direction
+
+    full_direction:
+        /(
+        NORTH |
+        NTH|
+        EAST  |
+        SOUTH |
+        STH|
+        WEST
+        )\ /x
+
+
+   abbrev_direction:
+        /(
+        N  |
+        NE |
+        NW |
+        E  |
+        S  |
+        SE |
+        SW |
+        W
+        )\ /x
 };
 
 #------------------------------------------------------------------------------
@@ -801,38 +860,31 @@ q{
 my $Australia =
 q{
     country:
-        /Australia ?/i |
-        /Aust\.? ?/i
+        /(AUSTRALIA|AUST|AU) ?/
 };
 
 my $Canada =
 q{
     country:
-        /Canada ?/i
+        /CANADA ?/
 };
 
 my $New_Zealand =
 q{
     country:
-        /New Zealand ?/i |
-        /NZ ?/
+        /(NEW ZEALAND|NZ) ?/
 };
 
 my $US =
 q{
     country:
-        /United States of America ?/i |
-        /United States ?/i |
-        /USA? ?/
+        /(UNITED STATES OF AMERICA|UNITED STATES|USA?) ?/
 };
 
 my $UK =
 q{
     country:
-        /Great Britain ?/i |
-        /United Kingdom ?/i |
-        /UK ?/ |
-        /GB ?/
+        /(GREAT BRITAIN|UNITED KINGDOM|UK|GB) ?/
 };
 
 my $non_matching =  q{ non_matching: /.*/ };
@@ -876,17 +928,17 @@ sub _create
     # Loop over all sub countries to create a grammar for all subcountry
     # combinations for this country. The grammar for Australia will look like
     #
-    # subcountry :  /NSW /i |
-    #               /QLD /i |
-    #               /NEW SOUTH WALES /i
-    #               /QUEENSLAND /i |
+    # subcountry :  /NSW / |
+    #               /QLD / |
+    #               /NEW SOUTH WALES /
+    #               /QUEENSLAND / |
 
     my @all_codes = $country->all_codes;
     my $last_code = pop(@all_codes);
 
     foreach my $code (@all_codes)
     {
-        $subcountry_grammar .= "\t/$code /i | \n";
+        $subcountry_grammar .= "\t/$code / | \n";
     }
     # No alternation character needed for last code
     $subcountry_grammar .= "\t/$last_code /\n";
@@ -901,8 +953,8 @@ sub _create
 
         foreach my $full_name (@all_full_names)
         {
-            $full_name = _clean_sub_country_name($full_name);
-            $subcountry_grammar .= "\t/$full_name /i |\n";
+            $full_name = uc(_clean_sub_country_name($full_name));
+            $subcountry_grammar .= "\t/$full_name / |\n";
         }
 
         $last_full_name = _clean_sub_country_name($last_full_name);
